use std::sync::Arc;

use axum::{
    http::{Request, StatusCode},
    middleware::Next,
    response::IntoResponse,
    Json,
};
use sqlx::SqlitePool;

use crate::{models::sessions::Session, types::error_code::ErrorCode};

use super::errors::ErrorResponse;

pub mod auth;
pub mod filter;
pub mod keys;
pub mod presence;
pub mod push;
pub mod room;
pub mod sync;
pub mod thirdparty;

async fn authentication_middleware<B>(mut req: Request<B>, next: Next<B>) -> impl IntoResponse {
    let db: &SqlitePool = req.extensions().get().unwrap();
    let auth_header = req
        .headers()
        .get(axum::http::header::AUTHORIZATION)
        .and_then(|header| header.to_str().ok());

    if auth_header.is_none() {
        return (
            StatusCode::FORBIDDEN,
            Json(ErrorResponse::new(
                ErrorCode::Forbidden,
                "Authorization Header not given",
                None,
            )),
        )
            .into_response();
    }

    let auth_header = auth_header.expect("Validated above");
    let idx = auth_header.find(' ');

    let idx = match idx {
        Some(idx) => idx,
        None => {
            return (
                StatusCode::FORBIDDEN,
                Json(ErrorResponse::new(
                    ErrorCode::Forbidden,
                    "Invalid Authorization Header",
                    None,
                )),
            )
                .into_response()
        }
    };

    let session = match Session::find_by_key(db, &auth_header[idx + 1..]).await {
        Ok(session) => session,
        Err(_) => {
            return (
                StatusCode::INTERNAL_SERVER_ERROR,
                Json(ErrorResponse::new(
                    ErrorCode::Unknown,
                    "Internal Server Error",
                    None,
                )),
            )
                .into_response()
        }
    };

    let session = match session {
        Some(session) => session,
        None => {
            return (
                StatusCode::FORBIDDEN,
                Json(ErrorResponse::new(ErrorCode::Forbidden, "Forbidden", None)),
            )
                .into_response()
        }
    };

    let device = match session.device(db).await {
        Ok(device) => device,
        Err(_) => {
            return (
                StatusCode::FORBIDDEN,
                Json(ErrorResponse::new(ErrorCode::Forbidden, "Forbidden", None)),
            )
                .into_response()
        }
    };

    let user = match device.user(db).await {
        Ok(user) => user,
        Err(_) => {
            return (
                StatusCode::FORBIDDEN,
                Json(ErrorResponse::new(ErrorCode::Forbidden, "Forbidden", None)),
            )
                .into_response()
        }
    };

    req.extensions_mut().insert(Arc::new(user));

    next.run(req).await.into_response()
}
